using System;
using System.Collections.Generic;

using Windows.Graphics.Imaging;
using Windows.Media;
using Windows.Media.Capture.Frames;

using Microsoft.UI.Xaml;
using Microsoft.UI.Xaml.Controls;
using Microsoft.UI.Xaml.Media.Imaging;

using CommunityToolkit.WinUI.Helpers;

namespace TestProject;

/// <summary>
///  
/// </summary>
public sealed partial class MainPage : Page
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Feld
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// ī޶ 
    /// </summary>
    private CameraHelper cameraHelper;

    /// <summary>
    ///   
    /// </summary>
    private VideoFrame currentVideoFrame;

    /// <summary>
    /// Ʈ Ʈ ҽ
    /// </summary>
    private SoftwareBitmapSource softwareBitmapSource;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region  - MainPage()

    /// <summary>
    /// 
    /// </summary>
    public MainPage()
    {
        InitializeComponent();

        Loaded                   += Page_Loaded;
        Unloaded                 += Page_Unloaded;
        this.captureButton.Click += captureButton_Click;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Private
    //////////////////////////////////////////////////////////////////////////////// Event

    #region  ε óϱ - Page_Loaded(sender, e)

    /// <summary>
    ///  ε óϱ
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private async void Page_Loaded(object sender, RoutedEventArgs e)
    {
        Loaded -= Page_Loaded;

        this.softwareBitmapSource = new SoftwareBitmapSource();

        this.currentImage.Source = this.softwareBitmapSource;

        IReadOnlyList<MediaFrameSourceGroup> mediaFrameSourceGroupList = await CameraHelper.GetFrameSourceGroupsAsync();

        if(this.cameraHelper == null)
        {
            this.cameraHelper = new CameraHelper();
        }

        CameraHelperResult result = await cameraHelper.InitializeAndStartCaptureAsync();

        if(result == CameraHelperResult.Success)
        {
            this.cameraHelper.FrameArrived += cameraHelper_FrameArrived!;

            this.frameSourceGroupComboBox.SelectionChanged += frameSourceGroupComboBox_SelectionChanged;

            this.frameSourceGroupComboBox.ItemsSource = mediaFrameSourceGroupList;

            this.frameSourceGroupComboBox.SelectedIndex = 0;
        }

        SetUIControls(result);
    }

    #endregion
    #region  ε óϱ - Page_Unloaded(sender, e)

    /// <summary>
    ///  ε óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private async void Page_Unloaded(object sender, RoutedEventArgs e)
    {
        Unloaded -= Page_Unloaded;

        if(this.frameSourceGroupComboBox != null)
        {
            this.frameSourceGroupComboBox.SelectionChanged -= frameSourceGroupComboBox_SelectionChanged;
        }

        if(this.cameraHelper != null)
        {
            this.cameraHelper.FrameArrived -= cameraHelper_FrameArrived;

            await this.cameraHelper.CleanUpAsync();

            this.cameraHelper = null!;
        }

        if(this.softwareBitmapSource != null)
        {
            this.softwareBitmapSource.Dispose();
        }
    }

    #endregion

    #region ī޶    óϱ - cameraHelper_FrameArrived(sender, e)

    /// <summary>
    /// ī޶    óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private void cameraHelper_FrameArrived(object sender, FrameEventArgs e)
    {
        this.currentVideoFrame = e.VideoFrame;
    }

    #endregion
    #region  ҽ ׷ ޺ ڽ   óϱ - frameSourceGroupComboBox_SelectionChanged(sender, e)

    /// <summary>
    ///  ҽ ׷ ޺ ڽ   óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private async void frameSourceGroupComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
        if(this.frameSourceGroupComboBox.SelectedItem is MediaFrameSourceGroup mediaFrameSourceGroup && this.cameraHelper != null)
        {
            this.cameraHelper.FrameSourceGroup = mediaFrameSourceGroup;

            CameraHelperResult result = await cameraHelper.InitializeAndStartCaptureAsync();

            SetUIControls(result);
        }
    }

    #endregion

    #region Capture video frame ư Ŭ óϱ - captureButton_Click(sender, e)

    /// <summary>
    /// Capture video frame ư Ŭ óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private async void captureButton_Click(object sender, RoutedEventArgs e)
    {
        SoftwareBitmap softwareBitmap = this.currentVideoFrame?.SoftwareBitmap;

        if(softwareBitmap != null)
        {
            if(softwareBitmap.BitmapPixelFormat != BitmapPixelFormat.Bgra8 || softwareBitmap.BitmapAlphaMode == BitmapAlphaMode.Straight)
            {
                softwareBitmap = SoftwareBitmap.Convert(softwareBitmap, BitmapPixelFormat.Bgra8, BitmapAlphaMode.Premultiplied);
            }
            if(this.softwareBitmapSource != null)
            {
                await softwareBitmapSource.SetBitmapAsync(softwareBitmap);
            }
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////// Function

    #region UI Ʈ ϱ - SetUIControls(result)

    /// <summary>
    /// UI Ʈ ϱ
    /// </summary>
    /// <param name="result">ī޶  </param>
    private void SetUIControls(CameraHelperResult result)
    {
        bool success = result == CameraHelperResult.Success;

        if(!success)
        {
            this.currentVideoFrame = null!;
        }

        this.inforBar.Title  = result.ToString();
        this.inforBar.IsOpen = !success;

        this.captureButton.IsEnabled = success;

        this.currentImage.Opacity = success ? 1 : 0.5;
    }

    #endregion
}